/*          C,
       L502  E502   
         .

         ,    PCI-Express  USB,
        ,    .
      Ethernet   IP-  
          ,   
         .
   ,      192.168.1.5  192.168.1.6,
      :
   x502_stream_read 192.168.1.5 192.168.1.6
             .
   
    ,     ..     
    .
      -    f_channels/f_ch_modes/f_ch_ranges.

         .
          Windows   CTRL+C  Linux.

             ,
       X502_ProcessData()     .

        Visual Studio 2008,     
   gcc  Linux  mingw  Windows  makefile    cmake (
       ).

         Visual Studio,     
   ( (Project) ->  (Properties) ->   (Configuration Properties)
   -> /++ ->  (General) ->    (Additional Include Directories))
    ,       x502api.h, l502api.h  e502api.h     
   ( (Project) ->  (Properties) ->   (Configuration Properties) ->
    (Linker) ->  (General) ->    (Additional Library Directories)).

   !!:      Visual Studio      (   SDK),
             
          UTF-8:
     (File) ->    (Advanced Save Options)...
       (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
       .

   */

#include "l502api.h"
#include "e502api.h"
#include "l502_bf_cmd_defs.h"


#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#else
#include <signal.h>
#include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>

/*     */
#define ADC_LCH_CNT  3

/*     */
#define ADC_FREQ          2000000
/*   (  ).  ADC_FREQ/ADC_LCH_CNT -    */
#define ADC_FRAME_FREQ    (ADC_FREQ/ADC_LCH_CNT)
/*     */
#define DIN_FREQ          2000000


#define TCP_CONNECTION_TOUT 5000


/*      */
#define READ_BLOCK_SIZE   4096*200
/*     () */
#define READ_TIMEOUT     2000


/*     */
static uint32_t f_channels[ADC_LCH_CNT] = {0,4,6};
/*     */
static uint32_t f_ch_modes[ADC_LCH_CNT] = {X502_LCH_MODE_DIFF, X502_LCH_MODE_DIFF, X502_LCH_MODE_DIFF};
/*     */
static uint32_t f_ch_ranges[ADC_LCH_CNT] = {X502_ADC_RANGE_10, X502_ADC_RANGE_10, X502_ADC_RANGE_10};



/*      */
static int f_out = 0;

#ifndef _WIN32
/*     Linux */
static void f_abort_handler(int sig) {
    f_out = 1;
}
#endif


/*        PCI-Express  USB 
 *        .
 *      IP-     
 * .
 *    ,     , 
 *  pdevrec_list,     (  
 *       X502_FreeDevRecordList()) 
 *    */
static uint32_t f_get_all_devrec(t_x502_devrec **pdevrec_list, uint32_t *ip_addr_list, unsigned ip_cnt) {
    int32_t fnd_devcnt = 0;
    uint32_t pci_devcnt = 0;
    uint32_t usb_devcnt = 0;

    t_x502_devrec *rec_list = NULL;

    /*       PCI  USB */
    L502_GetDevRecordsList(NULL, 0, 0, &pci_devcnt);
    E502_UsbGetDevRecordsList(NULL, 0, 0, &usb_devcnt);

    if ((pci_devcnt + usb_devcnt + ip_cnt) != 0) {
        /*          */
        rec_list = malloc((pci_devcnt + usb_devcnt + ip_cnt) * sizeof(t_x502_devrec));

        if (rec_list != NULL) {
            unsigned i;
            /*     L502,    pci_devcnt */
            if (pci_devcnt!=0) {
                int32_t res = L502_GetDevRecordsList(&rec_list[fnd_devcnt], pci_devcnt, 0, NULL);
                if (res >= 0) {
                    fnd_devcnt += res;
                }
            }
            /*     E502,   USB,    */
            if (usb_devcnt!=0) {
                int32_t res = E502_UsbGetDevRecordsList(&rec_list[fnd_devcnt], usb_devcnt, 0, NULL);
                if (res >= 0) {
                    fnd_devcnt += res;
                }
            }

            /*      ip- */
            for (i=0; i < ip_cnt; i++) {
                if (E502_MakeDevRecordByIpAddr(&rec_list[fnd_devcnt], ip_addr_list[i],0, TCP_CONNECTION_TOUT) == X502_ERR_OK) {
                    fnd_devcnt++;
                }
            }
        }
    }

    if (fnd_devcnt != 0) {
        /*      ,       */
        *pdevrec_list = rec_list;
    } else {
        *pdevrec_list = NULL;
        free(rec_list);
    }

    return fnd_devcnt;
}


static t_x502_hnd f_dev_select_open(int argc, char** argv) {
    t_x502_hnd hnd = NULL;
    uint32_t fnd_devcnt,i, dev_ind;
    t_x502_devrec *devrec_list = NULL;    
    uint32_t *ip_addr_list = NULL;
    uint32_t ip_cnt = 0;

    /*     ,  ,    
       ip-  . */
    if (argc > 1) {
        ip_addr_list = malloc((argc-1) * sizeof(ip_addr_list[0]));
        if (ip_addr_list == NULL) {
            fprintf(stderr, "  !\n");
        } else {
            for (i=1; (int)i < argc; i++) {
                int a[4];
                if (sscanf(argv[i], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])==4) {
                    ip_addr_list[ip_cnt++] = ((a[0] & 0xFF) << 24) |
                                             ((a[1] & 0xFF) << 16) |
                                             ((a[2] & 0xFF) <<  8) |
                                             (a[3] & 0xFF);
                }
            }
        }
    }

    /*      */
    fnd_devcnt = f_get_all_devrec(&devrec_list, ip_addr_list, ip_cnt);

    if (fnd_devcnt == 0) {
        printf("    \n");
    } else {
        /*      */
        printf("  :\n");
        for (i=0; i < fnd_devcnt; i++) {
            printf("  %d: %s, %-9s", i, devrec_list[i].devname,
                   devrec_list[i].iface == X502_IFACE_PCI ? "PCI/PCIe" :
                   devrec_list[i].iface == X502_IFACE_USB ? "USB" :
                   devrec_list[i].iface == X502_IFACE_ETH ? "Ethernet" : "?");

            /*      IP-    
                  .    location
                     */
            if (devrec_list[i].iface != X502_IFACE_ETH) {
                printf(". : %s\n", devrec_list[i].serial);
            } else {
                printf(": %s\n", devrec_list[i].location);
            }
        }

        /*           */
        printf("  ,     ( 0  %d)\n", fnd_devcnt-1);
        fflush(stdout);
        scanf("%d", &dev_ind);

        if (dev_ind >= fnd_devcnt) {
            printf("   ...\n");
        } else {
            /*     -   */
            hnd = X502_Create();
            if (hnd==NULL) {
                fprintf(stderr, "   !");
            } else {
                /*       */
                int32_t err = X502_OpenByDevRecord(hnd, &devrec_list[dev_ind]);
                if (err != X502_ERR_OK) {
                    fprintf(stderr, "    : %s!", X502_GetErrorString(err));
                    X502_Free(hnd);
                    hnd = NULL;
                }
            }
        }

        /*       */
        X502_FreeDevRecordList(devrec_list, fnd_devcnt);
        /*     */
        free(devrec_list);
    }

    /*     IP- (  ) */
    free(ip_addr_list);

    return hnd;
}


/*    */
int32_t f_setup_params(t_x502_hnd hnd) {
    int32_t err = X502_ERR_OK, i;

    /*      */
    err = X502_SetLChannelCount(hnd, ADC_LCH_CNT);
    for (i=0; (i < ADC_LCH_CNT) && (err == X502_ERR_OK); i++)
        err = X502_SetLChannel(hnd, i, f_channels[i], f_ch_modes[i], f_ch_ranges[i], 0);

    /*         */
    if (err == X502_ERR_OK) {
        double f_adc = ADC_FREQ, f_frame = ADC_FRAME_FREQ, f_din = DIN_FREQ;
        err = X502_SetAdcFreq(hnd, &f_adc, &f_frame);
        if (err == X502_ERR_OK)
            err = X502_SetDinFreq(hnd, &f_din);
        if (err == X502_ERR_OK) {
            /*     -     */
            printf(" :\n       = %0.0f\n"
                "      .  = %0.0f\n       = %0.0f\n",
                f_adc, f_frame, f_din);
        }
    }

    /*     */
    if (err == X502_ERR_OK)
        err = X502_Configure(hnd, 0);

    /*    */
    if (err == X502_ERR_OK) {
        err = X502_StreamsEnable(hnd, X502_STREAM_ADC | X502_STREAM_DIN);
    }

    return err;
}

int main(int argc, char** argv) {
    int32_t err = X502_ERR_OK;
    uint32_t ver;
    t_x502_hnd hnd = NULL;
#ifndef _WIN32
    struct sigaction sa;
    /*   Linux      ,
           */
    sa.sa_handler = f_abort_handler;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGABRT, &sa, NULL);
#endif
#ifdef _WIN32
    /*         Windows  CP1251    OEM */
    setlocale(LC_CTYPE, "");
#endif
    /*    */
    ver = X502_GetLibraryVersion();
    printf(" : %d.%d.%d\n", (ver >> 24)&0xFF, (ver>>16)&0xFF, (ver>>8)&0xFF);

    /**********     ,     ******************/
    hnd = f_dev_select_open(argc, argv);

    /**********************************    **************************/
    /*          -   */
    if (hnd != NULL) {
        /*   */
        t_x502_info info;
        err = X502_GetDevInfo(hnd, &info);
        if (err != X502_ERR_OK) {
            fprintf(stderr, "     : %s!", X502_GetErrorString(err));
        } else {
            /*    */
            printf("    :\n");
            printf("            : %s\n", info.serial);
            printf("               : %s\n", info.devflags & X502_DEVFLAGS_DAC_PRESENT ? "" : "");
            printf("  BlackFin        : %s\n", info.devflags & X502_DEVFLAGS_BF_PRESENT ? "" : "");
            printf("  : %s\n", info.devflags & X502_DEVFLAGS_GAL_PRESENT ? "" : "");
            printf("  .     : %s\n", info.devflags & X502_DEVFLAGS_INDUSTRIAL ? "" : "");
            printf("  . PCI/PCIe: %s\n", info.devflags & X502_DEVFLAGS_IFACE_SUPPORT_PCI ? "" : "");
            printf("  . USB     : %s\n", info.devflags & X502_DEVFLAGS_IFACE_SUPPORT_USB ? "" : "");
            printf("  . Ethernet: %s\n", info.devflags & X502_DEVFLAGS_IFACE_SUPPORT_ETH ? "" : "");
            printf("               : %d.%d\n", (info.fpga_ver >> 8) & 0xFF, info.fpga_ver & 0xFF);
            printf("  PLDA             : %d\n", info.plda_ver);
            if (info.mcu_firmware_ver != 0) {
                printf("   ARM     : %d.%d.%d.%d\n",
                       (info.mcu_firmware_ver >> 24) & 0xFF,
                       (info.mcu_firmware_ver >> 16) & 0xFF,
                       (info.mcu_firmware_ver >>  8) & 0xFF,
                       info.mcu_firmware_ver & 0xFF);
            }
        }

        {
            static uint32_t rx_data[L502_BF_CMD_DATA_SIZE_MAX];
            err = X502_BfExecCmd(hnd, 0x800B, 0, NULL, 0, rx_data, sizeof(rx_data)/sizeof(rx_data[0]), 0, NULL);
        }


        err = X502_BfLoadFirmware(hnd, "c:/PRJ/tmp/test_driver/Release/l502-reg.ldr");


        if (err == X502_ERR_OK) {
            /*    */
            err = f_setup_params(hnd);
            if (err != X502_ERR_OK)
                fprintf(stderr, "  : %s!", X502_GetErrorString(err));
        }


        /*   - */
        if (err == X502_ERR_OK) {
            err = X502_StreamsStart(hnd);
            if (err != X502_ERR_OK)
                fprintf(stderr, "   : %s!\n", X502_GetErrorString(err));
        }


        if (err == X502_ERR_OK) {
            int block;
            int32_t stop_err;

            printf("  .    %s\n",
#ifdef _WIN32
                   " "
#else
                   "CTRL+C"
#endif
                   );
            fflush(stdout);



            for (block = 0; (err == X502_ERR_OK) && !f_out; block++) {
                int32_t rcv_size;
                uint32_t adc_size, din_size;
                
                /*      */
                static uint32_t rcv_buf[READ_BLOCK_SIZE];
                static double   adc_data[READ_BLOCK_SIZE];
                static uint32_t din_data[READ_BLOCK_SIZE];

                /*   ( ) */
                rcv_size = X502_Recv(hnd, rcv_buf, READ_BLOCK_SIZE, READ_TIMEOUT);
                /*      */
                if (rcv_size < 0) {
                    err = rcv_size;
                    fprintf(stderr, "  : %s\n", X502_GetErrorString(err));
                } else if (rcv_size > 0) {
                    uint32_t first_lch;
                    /*    ,        */
                    X502_GetNextExpectedLchNum(hnd, &first_lch);

                    adc_size = sizeof(adc_data)/sizeof(adc_data[0]);
                    din_size = sizeof(din_data)/sizeof(din_data[0]);

                    /*   ,         */
                    err = X502_ProcessData(hnd, rcv_buf, rcv_size, X502_PROC_FLAGS_VOLT,
                                           adc_data, &adc_size, din_data, &din_size);
                    if (err != X502_ERR_OK) {
                        fprintf(stderr, "  : %s\n", X502_GetErrorString(err));
                    } else {
                        uint32_t lch;

                        printf(" %3d.    =%d,   =%d\n",
                               block, adc_size, din_size);    
                        /*     -    */
                        if (din_size != 0)
                            printf("    din_data = 0x%05X\n", din_data[0]);

                        /*      .   
                               ,     
                                   */
                        for (lch=0; lch < ADC_LCH_CNT; lch++) {
                            /*    ,    :
                                    ,     */
                            uint32_t pos = lch >= first_lch ? lch - first_lch : ADC_LCH_CNT-first_lch + lch;
                            if (pos <= adc_size) {
                                printf("    lch[%d]=%6.4f\n", lch, adc_data[pos]);
                            } else {
                                printf("    lch[%d]= ---- \n", lch);
                            }
                        }
                        printf("\n");
                        fflush(stdout);
                    }
                }
#ifdef _WIN32
                /*      */
                if (err == X502_ERR_OK) {
                    if (_kbhit())
                        f_out = 1;
                }
#endif
            }

            /*     (  ,   ) */
            stop_err = X502_StreamsStop(hnd);
            if (stop_err != X502_ERR_OK) {
                fprintf(stderr, "   : %s\n", X502_GetErrorString(err));
                if (err == X502_ERR_OK)
                    err = stop_err;
            } else {
                printf("   \n");
            }
        }

        /*     */
        X502_Close(hnd);
        /*   */
        X502_Free(hnd);
    }
    return err;
}
